/// \file console_input.ino
/// \brief User console interface for debugging using a host computer and the
/// default default serial port on an Arduino.

// Copyright (c) 2015-2017, Garth Zeglin. All rights reserved. Licensed under
// the terms of the BSD 3-clause license as included in LICENSE.

// ================================================================================
// Global state variables for the module.

/// The maximum message line length.
const int MAX_LINE_LENGTH = 80;
 
/// The maximum number of tokens in a single message.
const int MAX_TOKENS = 10;

// ================================================================================
// Utility functions

/// Convenience wrapper on strcmp for clarity of code.  Returns true if strings are identical.
static inline int string_equal(char *str1, char *str2) 
{
  return !strcmp(str1, str2);
}

// ================================================================================
/// Process a console input message.  This function should be customized for the
/// particular application to handle whatever user interaction or command input
/// is necessary.  The message has already been separated into a list of strings,
/// each holding one token split on the whitespace:
///   \param argc   number of argument tokens
///   \param argv   array of pointers to strings, one per token

void parse_console_input(int argc, char *argv[])
{
  // Interpret the first token as a command symbol.
  char *command = argv[0];

  // -- process zero-argument commands ---------------------------
  if (argc == 1) {
    if (string_equal(command, (char *) "report")) {
      Serial.println("Status report goes here.");
    }
    else if (string_equal(command, (char *) "reset")) {
      // do something to reset system here...
      Serial.println("Reset received.");
    }
    else if (string_equal(command, (char *) "time")) {
      // respond with the millisecond clock
      Serial.print("millis ");
      Serial.println(millis());
    }
    else {
      Serial.println("Unrecognized command.");
    }    
  }

  // -- process one-argument commands ---------------------------
  else if (argc == 2) {
    int value = atoi(argv[1]);
    if (string_equal(command, (char *) "led")) {
      // set the hardware LED output
      digitalWrite(LED_BUILTIN, value);
    }
    else {
      Serial.println("Unrecognized command.");
    }    
  }
  else {
    Serial.println("Unrecognized command.");
  }
}

// ================================================================================
/// Polling function to process messages arriving over the serial port.  Each
/// iteration through this polling function processes at most one character.  It
/// records the input message line into a buffer while simultaneously dividing it
/// into 'tokens' delimited by whitespace.  Each token is a string of
/// non-whitespace characters, and might represent either a symbol or an integer.
/// Once a message is complete, parse_console_input() is called.

void poll_console_input(unsigned long elapsed)
{
  static char input_buffer[ MAX_LINE_LENGTH ];   // buffer for input characters
  static char *argv[MAX_TOKENS];                 // buffer for pointers to tokens
  static int chars_in_buffer = 0;  // counter for characters in buffer
  static int chars_in_token = 0;   // counter for characters in current partially-received token (the 'open' token)
  static int argc = 0;             // counter for tokens in argv
  static int error = 0;            // flag for any error condition in the current message

  (void) elapsed;  // silence warnings about unused parameter

  // Check if at least one byte is available on the serial input.
  if (Serial.available()) {
    int input = Serial.read();

    // If the input is a whitespace character, end any currently open token.
    if (isspace(input)) {
      if (!error && chars_in_token > 0) {
	if (chars_in_buffer == MAX_LINE_LENGTH) error = 1;
	else {
	  input_buffer[chars_in_buffer++] = 0;  // end the current token
	  argc++;                               // increase the argument count
	  chars_in_token = 0;                   // reset the token state
	}
      }

      // If the whitespace input is an end-of-line character, then pass the message buffer along for interpretation.
      if (input == '\r' || input == '\n') {

	// if the message included too many tokens or too many characters, report an error
	if (error) Serial.println("error: excessive input.");

	// else process any complete message
	else if (argc > 0) parse_console_input(argc, argv); 

	// reset the full input state
	error = chars_in_token = chars_in_buffer = argc = 0;                     
      }
    }

    // Else the input is a character to store in the buffer at the end of the current token.
    else {
      // if beginning a new token
      if (chars_in_token == 0) {

	// if the token array is full, set an error state
	if (argc == MAX_TOKENS) error = 1;

	// otherwise save a pointer to the start of the token
	else argv[ argc ] = &input_buffer[chars_in_buffer];
      }

      // the save the input and update the counters
      if (!error) {
	if (chars_in_buffer == MAX_LINE_LENGTH) error = 1;
	else {
	  input_buffer[chars_in_buffer++] = input;
	  chars_in_token++;
	}
      }
    }
  }
}
// ================================================================================
