/// \file melody_player.ino
/// \brief Demo melody player module for EventBusyBox illustrating event-loop programming.

// \copyright Copyright (c) 2017, Garth Zeglin. All rights reserved. Licensed
// under the terms of the BSD 3-clause license as included in LICENSE.

#include "pitch_table.h"

// ================================================================================
// Global state variables for the module.

/// Pin output assignment for the speaker.
const int speakerPin = 7;

/// Whole-note duration in microseconds.
const long melody_whole_duration = 2000000L;  

/// Flag to indicate melody playing is underway.
bool melody_playing = false;

/// Time remaining in microseconds before the next note onset.
long melody_player_timer = 0;

/// Index into the melody note array for the next note to play.
int melody_player_next_note = 0;
  
/// A data structure for specifying each note in the melody by a combination of
/// pitch and duration.  A zero pitch and duration specifies the end of the melody.
/// N.B. this uses four bytes per note, not a very efficient use of memory, but
/// adequate for short sequences.
struct melody_note_t {
  /// Integer frequency value for a musical note.
  int pitch;
  
  /// Note length as a fraction of the measure tempo, e.g. 1 is a whole note, 4
  /// is a quarter note.
  int duration;
};

/// A C major scale, one quarter note per pitch, specified using pitch and duration.
struct melody_note_t melody[] = {
  {NOTE_C4, 4},
  {NOTE_D4, 4},
  {NOTE_E4, 4},
  {NOTE_F4, 4},
  {NOTE_G4, 4},
  {NOTE_A4, 4},
  {NOTE_B4, 4},
  {NOTE_C5, 4},
  {NOTE_END, 0}
};

// ================================================================================
/// Asynchronous start function to initiate playing a melody from another module
/// or reset a currently playing melody.
void start_melody(void)
{
  melody_playing = true;
  melody_player_timer = 0;
  melody_player_next_note = 0;
}
  
// ================================================================================
/// Configuration function to call from setup().
void setup_melody_player(void)
{
  /// Use the speaker pin for tone() output.
  pinMode(speakerPin, OUTPUT);

  /// For demo purposes, always begin by playing the melody once.
  start_melody();
}

// ================================================================================
/// Update function to poll from loop().
void poll_melody_player(unsigned long interval)
{
  if (melody_playing) {
    // Test whether to update the output.
    melody_player_timer -= interval;

    // The interval has elapsed once the timer variable reaches zero or overruns
    // into negative values:
    if (melody_player_timer <= 0) {

      // Fetch the next note and duration.
      int pitch    = melody[melody_player_next_note].pitch;
      int duration = melody[melody_player_next_note].duration;
      
      // Test if the melody is done.
      if (pitch <= 0) {
	noTone(speakerPin);
	melody_playing = false;
	
      } else {
	// Calculate the note duration in microseconds and update the timer.
	long note_duration = melody_whole_duration / duration;
	melody_player_timer += note_duration;

	// Start generating the next pitch.
	tone(speakerPin, pitch);

	// Advance to the next note for the next cycle.
	melody_player_next_note++;

	// Generate a debugging stream.
	Serial.print("pitch ");
	Serial.println(pitch);
      }
    }
  }
}
// ================================================================================



