/// \file servo_animation.ino
/// \brief Demo of an animation state machine using the servo linear
/// interpolation.  This module performs a fixed sequence of servo moves over
/// time, using the event-driven non-blocking style of EventBusyBox.  The actual
/// servo output uses poll_servo_sweeper().

// \copyright Copyright (c) 2017, Garth Zeglin. All rights reserved. Licensed
// under the terms of the BSD 3-clause license as included in LICENSE.

// ================================================================================
// Global state variables for the module.

/// Time remaining in microseconds before the next servo update.
long servo_animation_timer = 0;

/// The number of microseconds betwen animation updates.
const long servo_animation_interval = 100000;

/// Current state of the animation state machine.
int servo_animation_state = 0;

/// Time elapsed in current state in microseconds.
long servo_animation_elapsed = 0;

// ================================================================================
/// Utility function to switch to a new animation state.  This provides a hook
/// for common code to execute on every transition.
void servo_animation_transition(int new_state)
{
  Serial.print("animation entering ");
  Serial.println(new_state);
  servo_animation_elapsed = 0;
  servo_animation_state = new_state;
}
	
// ================================================================================
/// Asynchronous start function to initiate an animation from another module.
void start_servo_animation(void)
{
  servo_animation_transition(1);
}

// ================================================================================
/// Configuration function to call from setup().
void setup_servo_animation(void)
{
  // Start one iteration of the sequence as a demo.
  start_servo_animation();
}

// ================================================================================
/// Update function to poll from loop().
void poll_servo_animation(unsigned long interval)
{
  // Keep track of the time spent in any given state.
  servo_animation_elapsed += interval;
  
  // Test whether to update the state machine.
  servo_animation_timer -= interval;

  // The interval has elapsed once the timer variable reaches zero or overruns
  // into negative values:
  if (servo_animation_timer <= 0) {

    // Reset the timer for the next sampling period.
    servo_animation_timer += servo_animation_interval;

    // Branch to the current state machine state and apply any output changes or
    // state updates.

    switch(servo_animation_state){

    case 0:
      // idle, do nothing
      break;

    case 1:
      // Begin the animation by issuing a movement target.
      start_servo_sweep(90.0, 45.0);
      servo_animation_transition(10);
      break;

    case 10:
      // Wait until the servo sweeper has reached the target before proceeding.
      if (is_servo_sweeper_move_done()) {
	start_servo_sweep(180.0, 90.0);
	servo_animation_transition(20);	
      }
      break;
      
    case 20:
      if (is_servo_sweeper_move_done()) {
	start_servo_sweep(0.0, 45.0);
	servo_animation_transition(30);	
      }
      break;

    case 30:
      if (is_servo_sweeper_move_done()) {
	start_servo_sweep(180.0, 30.0);
	servo_animation_transition(40);	
      }
      break;

    case 40:
      if (is_servo_sweeper_move_done()) {
	start_servo_sweep(0.0, 30.0);
	servo_animation_transition(50);	
      }
      break;
      
    case 50:
      if (is_servo_sweeper_move_done()) {
	Serial.println("animation pausing");
	servo_animation_transition(60);
      }
      break;
      
    case 60:
      // Wait until an interval has passed.
      if (servo_animation_elapsed > 5000000L) {
	Serial.println("animation pause complete");
	servo_animation_transition(1);
      }
      break;
      
    default:
      Serial.println("animation invalid state");
      servo_animation_transition(0);
      break;
    }

    
  }
}
// ================================================================================
