/// \file servo_sweeper.ino
/// \brief Demo of servo linear interpolation for EventBusyBox illustrating
/// event-loop programming.  This module moves a hobby servo at an arbitrary
/// linear speed toward a position target.

// \copyright Copyright (c) 2016-2017, Garth Zeglin. All rights
// reserved. Licensed under the terms of the BSD 3-clause license as included in
// LICENSE.

// ================================================================================
// Import libraries.
#include <Servo.h> 

// ================================================================================
// Global state variables for the module.

/// Pin assigment for the servo command output.
const int SERVO_PIN = 9;

/// The number of microseconds between servo updates.
const long servo_sweeper_interval = 50000;

/// Instance of a Servo control object.  The Servo C++ class is defined in the
/// Servo library.
Servo sweeper_servo;

/// Time remaining in microseconds before the next servo update.
long servo_sweeper_timer = 0;

/// The most recently commanded servo angle in degrees, stored as a float for
/// finer speed precision.
float servo_sweeper_angle = 0.0;

/// The current servo target.  This is stored as a float to match servo_sweeper_angle.
float servo_sweeper_target = 0;

/// The current sweep step per update, in degrees.
float servo_sweeper_step = 0.0;

// ================================================================================
/// Asynchronous start function to initiate a linear servo motion.  This may be
/// called from another module.  to initiate a movement from the current servo
/// position to the specified target, emitting servo updates at a constant rate.
/// This function returns immediately, and the actual movement is performed by
/// update_servo_sweeper().
///  \param target  desired angle in degrees
///  \param speed   desired speed in degrees/sec

void start_servo_sweep(int target, float speed)
{
  // Save the target position.
  servo_sweeper_target = target;

  // Compute the size of each step in degrees.  Note the use of float to capture
  // fractional precision.  The constant converts speed units from milliseconds
  // to seconds:   deg/step = (deg/sec) * (sec/microseconds) * (microseconds/step)
  servo_sweeper_step = speed * 0.000001 * servo_sweeper_interval;
}  

// ================================================================================
/// Return a boolean indication of whether the servo has reached the target
/// position or not.  Note this only considers the commanded output; the servo
/// can physically lag behind the target.
bool is_servo_sweeper_move_done(void)
{
  return (servo_sweeper_angle == servo_sweeper_target);
}

// ================================================================================
/// Configuration function to call from setup().
void setup_servo_sweeper(void)
{
  // Initialize the Servo object to use the given pin for output.
  sweeper_servo.attach(SERVO_PIN);
}

// ================================================================================
/// Update function to poll from loop().
void poll_servo_sweeper(unsigned long interval)
{
  // Test whether to update the output.
  servo_sweeper_timer -= interval;

  // The interval has elapsed once the timer variable reaches zero or overruns
  // into negative values:
  if (servo_sweeper_timer <= 0) {

    // Reset the timer for the next sampling period.
    servo_sweeper_timer += servo_sweeper_interval;

    // Update the command value.  This correctly handles the case where the servo has already reached the target.
    if (servo_sweeper_target >= servo_sweeper_angle) {
      // Apply movement in the positive direction.
      servo_sweeper_angle += servo_sweeper_step;

      // If the movement would exceed the target, reset to the exact value.
      if (servo_sweeper_angle > servo_sweeper_target) servo_sweeper_angle = servo_sweeper_target;

    } else {
      // Else apply movement in the negative direction.
      servo_sweeper_angle -= servo_sweeper_step;

      // If the movement would exceed the target, reset to the exact value.
      if (servo_sweeper_angle < servo_sweeper_target) servo_sweeper_angle = servo_sweeper_target;
    }

    // Update the servo with the new position.
    sweeper_servo.write(servo_sweeper_angle);
  }
}
// ================================================================================

