/// \file speed_stepper.ino
/// \copyright Copyright (c) 2017, Garth Zeglin. All rights reserved. Licensed under the terms of the BSD 3-clause license as included in LICENSE.
/// \brief Stepper motor speed control for EventBusyBox illustrating event-loop programming.

/// \details Generate step pulses at controlled speeds for a continuously rotating stepper motor.

const int ledPin   = 13;  // onboard LED connected to digital pin 13

void setup() 
{
  Serial.begin(115200);
  pinMode(ledPin, OUTPUT);    // sets the LED pin as output
}

/****************************************************************/
// state variables for a rate controller
int next_step_output = HIGH;
unsigned long step_interval = 100000.0;
long step_timer = 0;
bool stepping = true;

void update_stepper (unsigned long interval)
{
  if (stepping) {
    // test whether to sample the input
    step_timer -= interval;
  
    if (step_timer <= 0) {

      // Reset the timer for the next sampling period.  Adding in the value helps
      // maintain precise timing in the presence of variation in the polling time,
      // e.g. if this sampling point was a little late, the next one will occur a
      // little sooner, maintaining the overall average.
      step_timer += step_interval;
      digitalWrite(ledPin, next_step_output);

      // toggle the pin state for the next cycle
      if (next_step_output == HIGH) next_step_output = LOW;
      else                          next_step_output = HIGH;
    }
  }
}

void set_stepper_speed(float steps_per_second)
{
  // For simplicity of the code, the STEP output is a square wave, so two cycles
  // will produce one motor driver step.  This computes the appropriate
  // half-cycle interval in microseconds.

  if (steps_per_second <= 0.0) {
    // avoid dividing by zero
    stepping = false;
  } else {  
    step_interval = (unsigned long) (500000.0 / steps_per_second);
    stepping = true;

    // reset the timer to shorten the current half-cycle to avoid problems when speeding up
    if (step_timer > step_interval) step_timer = step_interval;
  }
}

/****************************************************************/

// Free-running (non-blocking) event loop in which all actions are conditioned
// on the clock or inputs, with no delay().

void loop()
{
  // The timestamp in microseconds for the last polling cycle, used to compute
  // the exact interval between output updates.
  static unsigned long last_update_clock = 0;
  
  // Read the microsecond clock.
  unsigned long now = micros();

  // Compute the time elapsed since the last poll.  This will correctly handle wrapround of
  // the 32-bit long time value given the properties of twos-complement arithmetic.
  unsigned long interval = now - last_update_clock;
  last_update_clock = now;

  // set the current step rate in steps/second as a function of elapsed time.
  set_stepper_speed(5e-7 * now);

  // Calculate stepper outputs.
  update_stepper(interval);
}
